@@ -4,7 +4,7 @@ use crate::annostorage::{Match, ValueSearch};
 use crate::errors::Result;
 use crate::serializer::{FixedSizeKeySerializer, KeySerializer};
 use crate::types::{AnnoKey, Annotation, NodeID};
-use crate::util::disk_collections::{DiskMap, EvictionStrategy};
+use crate::util::disk_collections::{DiskMap, EvictionStrategy, DEFAULT_MAX_NUMBER_OF_TABLES};
 use crate::util::{self, memory_estimation};
 use core::ops::Bound::*;
 use rand::seq::IteratorRandom;
@@ -72,8 +72,8 @@ where
 /// [x Bits item ID][64 Bits symbol ID]
 /// ```
 fn create_by_container_key<T: FixedSizeKeySerializer>(item: T, anno_key_symbol: usize) -> Vec<u8> {
-    let mut result: Vec<u8> = Vec::from(item.create_key());
-    result.extend_from_slice(&anno_key_symbol.create_key());
+    let mut result: Vec<u8> = item.create_key().to_vec();
+    result.extend(anno_key_symbol.create_key());
     result
 }
 
@@ -92,7 +92,7 @@ fn create_by_anno_qname_key<T: FixedSizeKeySerializer>(
     anno_value: &str,
 ) -> Vec<u8> {
     // Use the qualified annotation name, the value and the node ID as key for the indexes.
-    let mut result: Vec<u8> = Vec::from(anno_key_symbol.create_key());
+    let mut result: Vec<u8> = anno_key_symbol.create_key().to_vec();
     for b in anno_value.as_bytes() {
         result.push(*b);
     }
@@ -120,10 +120,15 @@ where
             let path_by_anno_qname = path.join("by_anno_qname.bin");
 
             let mut result = AnnoStorageImpl {
-                by_container: DiskMap::new(Some(&path_by_container), EvictionStrategy::default())?,
+                by_container: DiskMap::new(
+                    Some(&path_by_container),
+                    EvictionStrategy::default(),
+                    DEFAULT_MAX_NUMBER_OF_TABLES,
+                )?,
                 by_anno_qname: DiskMap::new(
                     Some(&path_by_anno_qname),
                     EvictionStrategy::default(),
+                    DEFAULT_MAX_NUMBER_OF_TABLES,
                 )?,
                 anno_key_symbols: SymbolTable::default(),
                 anno_key_sizes: BTreeMap::new(),
@@ -311,10 +316,6 @@ where
         self.by_container
             .insert(by_container_key, anno.val.clone().into())?;
 
-        if self.by_container.number_of_disk_tables() > 7 {
-            self.by_container.compact()?;
-        }
-
         // To save some space, insert an empty array as a marker value
         // (all information is part of the key already)
         self.by_anno_qname.insert(
@@ -322,10 +323,6 @@ where
             true,
         )?;
 
-        if self.by_anno_qname.number_of_disk_tables() > 7 {
-            self.by_anno_qname.compact()?;
-        }
-
         if !already_existed {
             // a new annotation entry was inserted and did not replace an existing one
             if let Some(largest_item) = self.largest_item.clone() {
@@ -885,10 +882,12 @@ where
             self.by_container = DiskMap::new(
                 Some(&location.join("by_container.bin")),
                 EvictionStrategy::default(),
+                DEFAULT_MAX_NUMBER_OF_TABLES,
             )?;
             self.by_anno_qname = DiskMap::new(
                 Some(&location.join("by_anno_qname.bin")),
                 EvictionStrategy::default(),
+                DEFAULT_MAX_NUMBER_OF_TABLES,
             )?;
         }
 
